# tcp-lib.pl
# Common functions for managing inetd.conf and services files

do '../web-lib.pl';
&init_config();



# @wildcards
# Defines an array with all the wildcards that
# can be used in the hosts.allow and hosts.deny files.
@wildcards = ("KNOWN","UNKNOWN","LOCAL","PARANOID","ALL","EXCEPT");



# @commands
# Defines an array with all the commands that
# can be used in the hosts.allow and hosts.deny files.
@commands = ("nice","keepalive","linger","severity","allow","deny","rfc931","setenv","umask","user","spawn","twist","banners");



# list_inets()
# Returns a list of service details handled by inetd. RPC services
# will have a name like foo/1 or bar/1-3, where the thing after the / is
# the version or versions supported. For each service, the list contains
#  line active? rpc? name type protocol wait user path|internal [args]
sub list_inets
{
local(@rv, $l);
$l = 0;
open(INET, $config{'inetd_conf_file'});
while(<INET>) {
	chop;
	if (/^(#+)/) { }
	else {
		if (/\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\/\S+|internal)\s*(.*)$/) {
			push(@rv, [ $l, $1, $2, $3, $4, $5, $6, $7]);
			}
		}
		$l++;
	}
        -
close(INET);
return @rv;
}


#restart_inetd()
#restarts the inetd daemon to use the changes made
#in the /etc/inetd.conf file
sub restart_inetd
{
$whatfailed = $text{'error_restart_inetd'};

$out = &backquote_logged("$config{restart_inetd_command} 2>&1");
if ($?) {
	# Failed to signal inetd
	&error($out);
	}
}


#rule_array(rule,times)
#puts the rule into an array separating the differents parts that compose a rule
#(daemon, client, command1,..,commandn).
sub rule_array
{
my ($rule,$veces) = @_;
@array = ();
$i = 0;
$desplazamiento = 0;
while ($i <= $veces) {
$tamano = length($rule);
$posicion = index($rule,":");
if ($posicion >= 0) {
	$cadena = substr($rule,$desplazamiento,$posicion);
	if ($cadena =~ /\s*(.+)/) { push (@array, ($1)); }
	$desplazamiento = $posicion + 1;
	$rule = substr ($rule, $desplazamiento, $tamano);
	$desplazamiento = 0;
}
else {
	if ($rule =~ /\s*(.+)/) { push (@array, ($1)); }	
}
$i++;
}
return @array;
}


#char_number
#returns the number of times that a character has been found
#on a string
sub char_number
{
my ($rule,$char) = @_;
$i = 0;
while ($rule =~ /$char/g) { $i ++; }
return $i;
}



#list_rules(file)
#Returns a list of the rule described in the access control
#files (/etc/hosts.allow, /etc/hosts.deny) handled by the
#TCP Wrappers daemon TCPD
sub list_rules
{
my ($file) = @_;
$continue = 0;
$line = 0;
@rv = ();
$thereisrule = 0;
open(ARCHIVO, "$file");
while(eof(ARCHIVO) == 0) {
	$linea = <ARCHIVO>;
	chop $linea;
	$line++;
	if ($linea =~ /^#/) {}
	else {
		if ($continue==0) {
			$begin = $line;
			$rule = "";
			$size = 1;
		}
		if ($continue==1) {
			if ($linea =~ /(\s*|\t*)(.+)/) {
				$linea = $2;
				}
		}
		if ($linea =~ /\\$/) {
			chop $linea;
			$rule = "$rule" . "$linea";
			$continue = 1;
			$size++;
		}
		else {
			$rule = "$rule" . "$linea";
			$continue = 0;
			if ($rule =~ /(.+):\s*(.+)/) {
				$thereisrule = 1;
				push(@rv,[$begin, $size, rule_array($rule,char_number($rule,":"))]);
                	}
		}
	 	
	}
}
close(ARCHIVO);
if ($thereisrule == 0) { push (@rv, ($thereisrule)); }
return @rv;
}	



#get_rule(file,line,size)
#Get a specific rule in the access control file handled by
# the TCP Wrappers daemon TCPD
sub get_rule
{
my ($file,$line,$size) = @_;
@rv = ();
$i = 1;
$continue = 0;
$rule = "";
open(ARCHIVO, "$file");
while($size != 0) {
	$linea = <ARCHIVO>;
	chop $linea;
	if (($i == $line) or ($continue == 1)) {
		if ($continue==1) {
			if ($linea =~ /(\s*|\t*)(.+)/) {
				$linea = $2;
				}
		}
		if ($linea =~ /\\$/) {
			chop $linea;
			$continue = 1;
		}
		$rule = "$rule"."$linea";
		$size = $size - 1;
	}
	$i++;
}
close(ARCHIVO);
if ($rule =~ /(.+):\s*(.+)/) {
	push(@rv,(rule_array($rule,char_number($rule,":"))));
}
return @rv;
}	



#get_service_inetd(line)
#Returns the line in which the service is specified
#inside the /etc/inetd.conf file
sub get_service_inetd
{
open(INET, $config{'inetd_conf_file'});
@archivo = <INET>;
close(INET);
return $archivo[$_[0]];
}



#service_interpreter(array)
#Interprets all the documentation specified in the daemon part
#of a rule in the /etc/hosts.allow and /etc/hosts.deny files
sub service_interpreter
{
my ($servpart) = @_;
@arr = ();
if ($servpart =~ /\s*(\S+)\s*(\S*)\s*(\S*)/) {
	if ($2 and $3) {
		push(@arr,(3, $1, $2, $3));
		}
	else {
		push(@arr,(1, $1));
		}
	}
return @arr;	
}	



#host_interpreter(array)
#Interprets all the documentation specified in the client part
#of a rule in the /etc/hosts.allow and /etc/hosts.deny files
sub host_interpreter
{
my ($hostpart) = @_;
@arr = ();
if ($hostpart =~ /\s*(.+)/) {
	$client = $1;
	if ($1 =~ /(\S+)\s*(\S*)\s*(\S*)/) {
		if ($1 eq $text{'wildcard_all'}) {
			push(@arr,(3, $1, $2, $3));
			}
		if ($1 =~ /(\S+)\/+(\S+)/) {
			push(@arr,(2, $1, $2));
			}
		if ($1 =~ /(\S+)@(\S+)/) {
			if ((in_array(@wildcards,$1) == 1) and (in_array(@wildcards,$2) == 1)) {
				push(@arr, (5, $1, $2));
				}
			else {
				push(@arr,(1, $client));
				}
			}	
		else {
			if (in_array(@wildcards,$1) == 1) {
				push(@arr,(4, $1));
				}
			else {	
				push(@arr,(1, $1));
				}
			}
		}
	}
return @arr;	
}	



# make_rule(array)
# Gets an array and transform it in a string like rule
# ready to be put in the /etc/hosts.allow and /etc/hosts.deny files
sub make_rule
{
my (@arr) = @_;
$length = @arr;
$rule = "";
$i = 1;
while ($i <= $length) {
	if ($i != 1) {
		$rule = $rule . ": ";
		}
	#if ($i > 3) {
	#	$rule = $rule . "\\" . "\n" . "\t";
	#	}
	$indice = $i - 1;
	$rule = $rule . $arr[$indice];
	$i++;
}
$rule = $rule . "\n";
return $rule;
} 		



# sub_array(array,items)
# Returns a sub_array with a length of
# items defined
sub sub_array
{
$length = @_;
$items = $_[$length - 1];
splice(@_, -1);
$length = @_;
$result = $length - $items;
splice(@_, $items, $result);
return @_;
}



# in_array(array,element)
# Searches an element inside an array
# Returns 1 if founded, 0 if not.
sub in_array
{
$length = @_;
$item = $_[$length - 1];
splice(@_, -1);
$length = @_;
$i = 0;
$find = 0;
while ($i < $length) {
	if ($_[$i] eq $item) {
		$find = 1;
		}
	$i++;
	}
return $find;
}


#same_array(arr1,arr2)
#This function checks if the two arrays are the same or not
sub same_array
{
my ($arr,$arr2) = @_;
$length1 = @arr;
$length2 = @arr2;
$i = 0;
if ($length1 == $length2) {
	$same = 1;
	while ($i < $length2) {
		if ($arr[$i] ne $arr2[$i]) { $same = 0; }
		$i++;
		}
        }
else { $same = 0; }
return $same;
}


#init_temporary_files()
#cleans all the temporary files that need to be used when
#a rule is added or modified
sub init_temporary_files
{
open (ARCHIVO, "> rule.tmp");
close (ARCHIVO);
open (ARCHIVO, "> commands.tmp");
close (ARCHIVO);
open (ARCHIVO, "> banner.tmp");
close (ARCHIVO)
}


#get_daemon(args)
#gets the name of the daemon of the respective service
sub get_daemon
{
if ($_[0] =~ /.+\/(\S+)/) { $value = $1; }
else {
	if ($_[0] =~ /(\S+)/) { $value = $1; }
}
return $value;
}



sub log_counter
{
$length = @_;
$item = $_[$length - 1];
splice(@_, -1);
$length = @_;
$i = 0;
$find = 0;
while ($i < $length) {
	if ($_[$i] =~ /$item/) { $find++; }
	$i++;
}
return $find;
}



sub log_sub_array
{
$length = @_;
$item = $_[$length - 1];
splice(@_, -1);
$length = @_;
$i = 0;
@array = ();
while ($i < $length) {
	if ($_[$i] =~ /$item/) { push (@array,$_[$i]); }
	$i++;
}
return @array;
}



sub fill_banners_dir
{
my($path,$daemon,$message) = @_;
@temp_array = &service_interpreter($daemon);
if ($temp_array[0] == 1) {
	if ($temp_array[1] eq $text{'wildcard_all'}) {
		@hlist = &list_inets();
		foreach $h (@hlist) {
			if ($h->[6] eq $config{'tcpd_path'}) {
				if ($h->[7]) {					
					$value = &get_daemon($h->[7]);
				}
				else { $value = $h->[1]; }
			$tempath = "> " . $path . $value;
			open (BANNER, $tempath);
			print BANNER $message;
			close (BANNER);
			}
		}		
	}
	else {
		$path = "> " . $path . $daemon;
		open (BANNER, $path);
		print BANNER $message;
		close (BANNER);
	}
}
else { }
}

